/*
Created by: X
http://www.createdbyx.com/

This work is licensed under a Creative Commons Attribution-Noncommercial-Share Alike 2.5 Canada License 
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/. 
Permissions beyond the scope of this license may be available at http://contact.createdbyx.com/.
*/
using CBXInput4XNA;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XNAGameCodeKit;

namespace PathFindingExample1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public partial class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        // Used for easy KB,Mouse,gamepad input
        private CBXInput4XNA.IUserInputService input;
        // used to draw lies on screen
        private XNAGameCodeKit.LineDrawing lineDrawer;
        // used to manage and draw a grid on screen
        private GridComponent grid;
        // hold weather or not the user is editing the grid tiles
        private bool editingGrid = true;
        // holds the size of each grid tile
        int cellSize = 16;
        // convienence component to drawing text to the screen
        private XNAGameCodeKit.DebugTextComponent debugText;
        // holds the start and end tile locations
        private Point startPosition;
        private Point endPosition;
       // grid colors
        Color fillColor = new Color(0, 0, 128, 32);
        Color fillSolidColor = new Color(0, 32, 128, 128);


        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            this.IsMouseVisible = true;

            graphics.PreferredBackBufferWidth = 1280;
            graphics.PreferredBackBufferHeight = 720;
            graphics.ApplyChanges();

            // create a input object and add it to the compoents list
            this.input = new UserInputService(this);
            this.Components.Add((IGameComponent)this.input);

            // setup the grid component
            this.grid = new GridComponent(this, graphics.PreferredBackBufferWidth / this.cellSize, graphics.PreferredBackBufferHeight / this.cellSize);
            this.Components.Add(this.grid);

            // setup the line drawing component
            this.lineDrawer = new LineDrawing(this);
            this.Components.Add(this.lineDrawer);

            // setup the text drawing system
            this.debugText = new DebugTextComponent(this, "CourierNew");
            this.Components.Add(this.debugText);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (this.input.KeyClick(Keys.Escape)) this.Exit();
            // check to randomize the grid
            if (this.input.KeyClick(Keys.R))
            {
                this.grid.RandomizeGrid();
                this.SolvePath();
            }
            // checked IFormatProvider changine editing modes
            if (this.input.KeyClick(Keys.Space)) this.editingGrid = !this.editingGrid;
            
            // check if editing grid tiles or positioning the start and end locations
            if (this.editingGrid)
            {
                this.DoEditGrid();
            }
            else
            {
                this.DoSetPoints();
            }

            // clear the output text and draw the new text info
            this.debugText.Clear();
            this.debugText.Print("Press R to randomize the grid");
            this.debugText.Print("Press Enter to solve path");
            this.debugText.Print("Press Spacebar to switch modes");
            this.debugText.Print();
            this.debugText.Print("Mode: " + (this.editingGrid ? "Editing grid" : "Setting Start and End positions"));
            if (this.editingGrid)
            {
                this.debugText.Print("Use left and right mouse buttons to draw or erase blocks");
            }
            else
            {
                this.debugText.Print("Use left and right mouse buttons to set start and end positions");
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// Positions and checks the start and end locations depending on what mouse button is pressed
        /// </summary>
        private void DoSetPoints()
        {
            Point pos = new Point(this.input.MouseX / this.cellSize, this.input.MouseY / this.cellSize);
            if (pos.X < 0) pos.X = 0;
            if (pos.Y < 0) pos.Y = 0;
            if (pos.X > this.grid.GridWidth - 1) pos.X = this.grid.GridWidth - 1;
            if (pos.Y > this.grid.GridHeight - 1) pos.Y = this.grid.GridHeight - 1;
            bool needsUpdate = false;
            if (this.input.IsLeftMouseButtonDown)
            {
                this.startPosition = pos;
                needsUpdate = true;
            }
            if (this.input.IsRightMouseButtonDown)
            {
                this.endPosition = pos;
                needsUpdate = true;
            }
            if (needsUpdate) this.SolvePath();
        }

        /// <summary>
        /// Created or deletes tiles depending on what mouse button is pressed
        /// </summary>
        private void DoEditGrid()
        {
            Point pos = new Point(this.input.MouseX / this.cellSize, this.input.MouseY / this.cellSize);
            if (pos.X < 0) pos.X = 0;
            if (pos.Y < 0) pos.Y = 0;
            if (pos.X > this.grid.GridWidth - 1) pos.X = this.grid.GridWidth - 1;
            if (pos.Y > this.grid.GridHeight - 1) pos.Y = this.grid.GridHeight - 1;
            bool needsUpdate = false;
            if (this.input.IsLeftMouseButtonDown)
            {
                this.grid.SetCell(pos.X, pos.Y, true);
                needsUpdate = true;
            }
            if (this.input.IsRightMouseButtonDown)
            {
                this.grid.SetCell(pos.X, pos.Y, false);
                needsUpdate = true;
            }
            if (needsUpdate) this.SolvePath();
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            // clear backbuffer
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // draw the grid
            for (int idx = 0; idx < this.grid.GridWidth - 1; idx++)
            {
                for (int idy = 0; idy < this.grid.GridHeight - 1; idy++)
                {
                    this.lineDrawer.FillRectange(idx * cellSize, idy * cellSize, cellSize, cellSize, this.grid.GetCell(idx, idy) ? fillSolidColor : fillColor);
                    this.lineDrawer.DrawRectange(idx * cellSize, idy * cellSize, cellSize, cellSize, Color.Blue);
                }
            }

            // draw start/end pos
            this.lineDrawer.FillRectange(this.startPosition.X * this.cellSize, this.startPosition.Y * this.cellSize, this.cellSize, this.cellSize, Color.Green);
            this.lineDrawer.FillRectange(this.endPosition.X * this.cellSize, this.endPosition.Y * this.cellSize, this.cellSize, this.cellSize, Color.Red);

            // draw the path
            this.DrawPath();

            base.Draw(gameTime);
        }
    }
}
