﻿/*
Created by: X
http://www.createdbyx.com/

This work is licensed under a Creative Commons Attribution-Noncommercial-Share Alike 2.5 Canada License 
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/. 
Permissions beyond the scope of this license may be available at http://contact.createdbyx.com/.
*/
using System;
using Microsoft.Xna.Framework;

namespace SLPathfindaz
{
    /// <summary>
    /// Defines extension methods that make it even easier to work with SLPathfindaz.
    /// </summary>
    public static class ExtensionMethods
    {
        /// <summary>
        /// Gets a path givin the start and destination locations.
        /// </summary>
        /// <param name="finder">Reference to a PathFinder object.</param>
        /// <param name="start">Start tile location.</param>
        /// <param name="destination">Destination tile location.</param>
        /// <returns>Returns an array of Point types contaiing the discovered path.</returns>
        public static Point[] GetPath(this PathFinder finder, Point start, Point destination)
        {
            return GetPath(finder, start.X, start.Y, destination.X, destination.Y);
        }

        /// <summary>
        /// Gets a path givin the start and destination locations.
        /// </summary>
        /// <param name="finder">Reference to a PathFinder object.</param>
        /// <param name="startX">Start x tile location.</param>
        /// <param name="startY">Start y tile location.</param>
        /// <param name="endX">Destination x tile location.</param>
        /// <param name="endY">Destination y tile location.</param>
        /// <returns>Returns an array of Point types contaiing the discovered path.</returns>
        public static Point[] GetPath(this PathFinder finder, int startX, int startY, int endX, int endY)
        {
            //List<Point> list = new List<Point>();
            int increment = 10000;
            Point[] list = new Point[increment];
            Point[] array;

            // start by giving the finder a start and end location
            finder.FindPath(startX, startY, endX, endY);
             
            // loop through until complete
            int index = 0;
            while (!finder.Complete)
            {
                list[index] = finder.GetNextWaypoint();
                index++;
                if (index >= list.Length)
                {
                    array = new Point[list.Length + increment];
                    Array.Copy(list, array, list.Length - increment);
                    list = array;
                }
            }

            // if still zero then no points wer added so return null
            if (index == 0) list = null;

            // reduce array size if needed
            if (list != null && index <= list.Length)
            {
                array = new Point[index];
                Array.Copy(list, array, index);
                list = array;
            }

            array = null;
            return list;
        }
    }
}
